@@ -22,25 +22,47 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import logging
 import os
-import subprocess
-import string
 import re
-import pty
-import warnings
-warnings.filterwarnings("ignore", "apt API not stable yet", FutureWarning)
-import apt_pkg
+import string
+import sys
 
-from PyKDE4.kdecore import *
-from PyKDE4.kdeui import *
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+
+from PyKDE4.kdeui import (
+    KApplication,
+    KIcon,
+    KMessageBox,
+    DesktopIcon,
+    )
+from PyQt4.QtCore import (
+    Qt,
+    SIGNAL,
+    QStringList,
+    QTimer,
+    )
+from PyQt4.QtGui import (
+    QDialog,
+    QHBoxLayout,
+    QTextCursor,
+    QTextEdit,
+    QTextOption,
+    )
 from PyQt4 import uic
 
-from DebPackage import DebPackage, Cache
+from apt.cache import Cache
+
 import gettext
-from GDebiCommon import GDebiCommon, utf8, _
-from KDEAptDialogs import *
+
+from .DebPackage import DebPackage
+from .GDebiCommon import GDebiCommon, utf8, _
+from .KDEAptDialogs import (
+    CacheProgressAdapter,
+    KDEDpkgInstallProgress,
+    KDEFetchProgressAdapter,
+    KDEInstallProgressAdapter,
+    )
+
 
 def __(catalog,str):
     return unicode(gettext.dgettext(catalog, str), 'UTF-8')
@@ -107,19 +129,23 @@ class DumbTerminal(QTextEdit):
             return
         # block signals so that we do not run into a recursion
         self._block = True
-        para = self.paragraphs() - 1
-        pos = self.paragraphLength(para)
+        #para = self.paragraphs() - 1
+        #pos = self.paragraphLength(para)
         self.moveCursor(QTextCursor.End)
         self._block = False
 
+
 class GDebiKDEDialog(QDialog):
     """Our main user interface, load from UI file"""
     def __init__(self, parent):
         QDialog.__init__(self, parent)
         loadUi("GDebiKDEDialog.ui", self)
 
+
 class GDebiKDE(GDebiCommon, GDebiKDEDialog):
-    def __init__(self,datadir,options,file="",parent = None,name = None,modal = 0,fl = 0):
+
+    def __init__(self, datadir, options, file="", parent=None, name=None,
+                 modal=0, fl=0):
         GDebiKDEDialog.__init__(self,parent)
         GDebiCommon.__init__(self,datadir,options,file)
         # load the icon
@@ -134,6 +160,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         self.tabWidget2.setTabText(2,_("Included Files"))
         self.cancelButton.setText(__("kdelibs","&Cancel"))
         self.installButton.setText(_("&Install Package"))
+        self.downloadButton.setText(_("&Download Package"))
         self.DetailsVersionLabel.setText(_("<b>Version:</b>"))
         self.DetailsMaintainerLabel.setText(_("<b>Maintainer:</b>"))
         self.DetailsPriorityLabel.setText(_("<b>Priority:</b>"))
@@ -143,6 +170,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         self.setDisabled(True)
         self.PackageProgressBar.setEnabled(True)
         self.detailsButton.hide()
+        self.downloadButton.hide()
         self.installButton.setIcon(KIcon("dialog-ok"))
         self.cancelButton.setIcon(KIcon("dialog-cancel"))
         self.show()
@@ -166,18 +194,19 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         self.PackageProgressBar.hide()
         self.connect(self.cancelButton, SIGNAL("clicked()"), self.cancelButtonClicked)
         self.connect(self.installButton, SIGNAL("clicked()"), self.installButtonClicked)
+        self.connect(self.downloadButton, SIGNAL("clicked()"), self.downloadButtonClicked)
         self.connect(self.detailsButton, SIGNAL("clicked()"), self.detailsButtonClicked)
 
-    def open(self, file):
+    def open(self, file, downloaded=False):
         """ load the .deb file """
-        res = GDebiCommon.open(self, file)
+        res = GDebiCommon.open(self, file, downloaded)
         if res == False:
             KMessageBox.error(self, '<b>' + self.error_header + '</b><br>' + self.error_body, self.error_header)
             return False
 
         # set name
-        self.setWindowTitle(_("Package Installer - %s") % self._deb.pkgName)
-        self.textLabel1_3.setText(self._deb.pkgName)
+        self.setWindowTitle(_("Package Installer - %s") % self._deb.pkgname)
+        self.textLabel1_3.setText(self._deb.pkgname)
 
         # set description
         buf = self.DecriptionEdit
@@ -215,16 +244,28 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
             buf.setText(_("No description is available"))
 
         # check deps
-        if not self._deb.checkDeb():
+        if not self._deb.check():
             self.installButton.setEnabled(False)
-            self.textLabel1_3_2.setText("<font color=\"#ff0000\"> Error: " + self._deb._failureString + "</font>")
+            self.textLabel1_3_2.setText(utf8("<font color=\"#ff0000\"> Error: " + self._deb._failure_string + "</font>"))
             self.detailsButton.hide()
             return False
 
+        # check provides
+        provides = self.compareProvides()
+        if provides:
+            self.installButton.setEnabled(False)
+            self.textLabel1_3_2.setText(utf8("<font color=\"#ff0000\"> Error: no longer provides " + ", ".join(provides) + "</font>"))    
+            return False
+
         # set version_info_{msg,title} strings
         self.compareDebWithCache()
 
-        if self._deb.compareToVersionInCache() == DebPackage.VERSION_SAME:
+        version_status = self._deb.compare_to_version_in_cache(use_installed=False)
+        if (version_status in (DebPackage.VERSION_SAME, DebPackage.VERSION_OUTDATED)):
+            if not self._deb.downloaded:
+                self.downloadButton.show()
+
+        if self._deb.compare_to_version_in_cache() == DebPackage.VERSION_SAME:
             #self.textLabel1_3_2.setText(_("Same version is already installed"))
             self.installButton.setText(_("&Reinstall Package"))
             self.installButton.setIcon(KIcon("view-refresh"))
@@ -234,7 +275,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
             #self.button_details.hide()
 
         # load changes into (self.install, self.remove, self.unauthenticated)
-        self.getChanges()
+        self.get_changes()
 
         if self.version_info_title != "" and self.version_info_msg != "":
             self.infoIcon.setPixmap(DesktopIcon("dialog-information"))
@@ -251,16 +292,16 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         self.DetailsMaintainer.setText(utf8(self._deb["Maintainer"]))
         self.DetailsPriority.setText(self._deb["Priority"])
         self.DetailsSection.setText(utf8(self._deb["Section"]))
-        self.DetailsSize.setText(self._deb["Installed-Size"] + " KB")
+        self.DetailsSize.setText(self._deb["Installed-Size"] + " KiB")
 
         # set filelist
         buf = self.IncFilesEdit
         buf.setText("\n".join(self._deb.filelist))
 
-        if not self._deb.checkDeb():
+        if not self._deb.check():
             self.installButton.setText(_("&Install Package"))
 
-        if self._deb.compareToVersionInCache() == DebPackage.VERSION_SAME:
+        if self._deb.compare_to_version_in_cache() == DebPackage.VERSION_SAME:
             self.installButton.setText(_("Re&install Package"))
 
     def cancelButtonClicked(self):
@@ -268,15 +309,16 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
 
     def detailsButtonClicked(self):
         changedList = QStringList()
-        (install, remove, unauthenticated) = self._deb.requiredChanges
+        (install, remove, unauthenticated) = self._deb.required_changes
         for i in install:
             changedList.append(_("To be installed: %s") % i)
         for r in remove:
             changedList.append(_("To be removed: %s") % r)
 
-        infoReport = KMessageBox.informationList(self,
-                      _("<b>To install the following changes are required:</b>"),
-                      changedList, _("Details"))
+        KMessageBox.informationList(
+            self,
+            _("<b>To install the following changes are required:</b>"),
+            changedList, _("Details"))
 
     def installButtonClicked(self):
         # if not root, start a new instance
@@ -285,8 +327,10 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
                 executable = os.path.curdir + "/gdebi-kde"
             else:
                 executable = "/usr/bin/gdebi-kde"
-            print "executable " + executable
-            os.execl("/usr/bin/kdesudo", "kdesudo", executable + " -n ", self._deb.file)
+            su_cmd = "/usr/bin/kdesudo"
+            if not os.access(su_cmd, os.X_OK):
+                su_cmd = "/usr/lib/kde4/libexec/kdesu"
+            os.execl(su_cmd, os.path.basename(su_cmd), executable, "--", "-n", self._deb.filename)
             self.kapp.exit()
 
         if not self.try_acquire_lock():
@@ -306,7 +350,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         if len(self.install) > 0 or len(self.remove) > 0:
             #if not self.acquire_lock():
             if not self.acquire_lock():
-                #self.show_alert(gtk.MESSAGE_ERROR, self.error_header, self.error_body)
+                #self.show_alert(Gtk.MessageType.ERROR, self.error_header, self.error_body)
                 KMessageBox.sorry(self, '<b>' + self.error_header + '</b><br>' + self.error_body, self.error_header)
                 return False
             fprogress = KDEFetchProgressAdapter(self.installDialog.installationProgress,
@@ -316,16 +360,17 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
                                                   self.installDialog.installingLabel,
                                                   self.installDialog)
             self.installDialog.konsole.setInstallProgress(iprogress)
-            errMsg = None
+            #errMsg = None
             try:
                 res = self._cache.commit(fprogress,iprogress)
-            except IOError, msg:
+            except IOError as msg:
                 res = False
-                errMsg = "%s" % msg
+                #errMsg = "%s" % msg
                 header = _("Could not download all required files")
                 body = _("Please check your internet connection or "
                             "installation medium.")
-            except SystemError, msg:
+            except SystemError as msg:
+                logging.warn("error: %s" % msg)
                 res = False
                 header = _("Could not install all dependencies")
                 body = _("Usually this is related to an error of the "
@@ -338,7 +383,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
 
         # install the package itself
         #self.label_action.set_markup("<b><big>"+_("Installing package file")+"</big></b>")
-        dprogress = KDEDpkgInstallProgress(self._deb.file,
+        dprogress = KDEDpkgInstallProgress(self._deb.filename,
                                              self.installDialog.installingLabel,
                                              self.installDialog.installationProgress,
                                              self.installDialog.konsole, self.installDialog)
@@ -349,9 +394,9 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         #self.button_deb_install_close.grab_default()
         self.installDialog.setWindowTitle(_("Installation finished"))
         if dprogress.exitstatus == 0:
-            self.installDialog.installingLabel.setText(_("<b>" + "Package '%s' was installed" + "</b>") % self._deb.pkgName)
+            self.installDialog.installingLabel.setText(_("<b>" + "Package '%s' was installed" + "</b>") % self._deb.pkgname)
         else:
-            self.installDialog.installingLabel.setText("<b>"+_("Failed to install package '%s'") % os.path.basename(self._deb.file)+"</b>")
+            self.installDialog.installingLabel.setText("<b>"+_("Failed to install package '%s'") % os.path.basename(self._deb.filename)+"</b>")
             self.installDialog.konsoleFrame.show()
         #self.statusbar_main.push(self.context,_("Installation complete"))
         # FIXME: Doesn't stop notifying
@@ -360,17 +405,22 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         # reopen the cache, reread the file, FIXME: add progress reporting
         #self._cache = Cache(self.cprogress)
         self._cache = Cache()
-        if self._cache._depcache.BrokenCount > 0:
+        if self._cache._depcache.broken_count > 0:
             header = _("Failed to completely install all dependencies")
             text = _("To fix this run 'sudo apt-get install -f' in a "
                          "terminal window.")
             self.errorReport = KMessageBox.error(self, header + text, header)
             sys.exit(1)
 
+    def downloadButtonClicked(self):
+        if self.download_package():
+            self.downloadButton.hide()
+
 class GDebiKDEInstall(QDialog):
     def __init__(self, parent):
         QDialog.__init__(self, parent)
         loadUi("GDebiKDEInstallDialog.ui", self)
+        self.setWindowTitle(_("Installing"))
         self.showDetailsButton.setText(__("libept","Show Details")) #FIXME check i18n
         self.closeButton.setText(__("kdelibs","&Close"))
         self.showDetailsButton.setIcon(KIcon("utilities-terminal"))
